#include "stdafx.h"
#include "HDUSTest.h"
#include "BasicWindow.h"




CBasicWindow::CBasicWindow()
{
	m_hwnd=NULL;
	m_WindowPosition.Left=0;
	m_WindowPosition.Top=0;
	m_WindowPosition.Width=0;
	m_WindowPosition.Height=0;
}


CBasicWindow::~CBasicWindow()
{
	Destroy();
}


void CBasicWindow::Destroy()
{
	if (m_hwnd!=NULL)
		::DestroyWindow(m_hwnd);
}


void CBasicWindow::SetPosition(int Left,int Top,int Width,int Height)
{
	if (m_hwnd!=NULL) {
		::MoveWindow(m_hwnd,Left,Top,Width,Height,TRUE);
	} else {
		m_WindowPosition.Left=Left;
		m_WindowPosition.Top=Top;
		m_WindowPosition.Width=Width;
		m_WindowPosition.Height=Height;
	}
}


void CBasicWindow::GetPosition(int *pLeft,int *pTop,int *pWidth,int *pHeight) const
{
	if (m_hwnd!=NULL) {
		if ((GetWindowStyle(m_hwnd)&WS_CHILD)!=0) {
			RECT rc;

			::GetWindowRect(m_hwnd,&rc);
			::MapWindowPoints(NULL,GetParent(m_hwnd),(LPPOINT)&rc,2);
			if (pLeft)
				*pLeft=rc.left;
			if (pTop)
				*pTop=rc.top;
			if (pWidth)
				*pWidth=rc.right-rc.left;
			if (pHeight)
				*pHeight=rc.bottom-rc.top;
		} else {
			WINDOWPLACEMENT wc;

			wc.length=sizeof(WINDOWPLACEMENT);
			::GetWindowPlacement(m_hwnd,&wc);
			if (pLeft)
				*pLeft=wc.rcNormalPosition.left;
			if (pTop)
				*pTop=wc.rcNormalPosition.top;
			if (pWidth)
				*pWidth=wc.rcNormalPosition.right-wc.rcNormalPosition.left;
			if (pHeight)
				*pHeight=wc.rcNormalPosition.bottom-wc.rcNormalPosition.top;
		}
	} else {
		if (pLeft)
			*pLeft=m_WindowPosition.Left;
		if (pTop)
			*pTop=m_WindowPosition.Top;
		if (pWidth)
			*pWidth=m_WindowPosition.Width;
		if (pHeight)
			*pHeight=m_WindowPosition.Height;
	}
}


void CBasicWindow::GetPosition(RECT *pPosition) const
{
	int Left,Top,Width,Height;

	GetPosition(&Left,&Top,&Width,&Height);
	::SetRect(pPosition,Left,Top,Left+Width,Top+Height);
}


void CBasicWindow::SetVisible(bool fVisible)
{
	if (m_hwnd!=NULL)
		::ShowWindow(m_hwnd,fVisible?SW_SHOW:SW_HIDE);
}


bool CBasicWindow::GetVisible() const
{
	return m_hwnd!=NULL && ::IsWindowVisible(m_hwnd)!=FALSE;
}


bool CBasicWindow::Invalidate(bool fErase)
{
	return m_hwnd!=NULL && ::InvalidateRect(m_hwnd,NULL,fErase);
}


bool CBasicWindow::Update()
{
	return m_hwnd!=NULL && ::UpdateWindow(m_hwnd);
}


bool CBasicWindow::MoveToMonitorInside()
{
	RECT rc;
	HMONITOR hMonitor;
	MONITORINFO mi;

	GetPosition(&rc);
	hMonitor=MonitorFromRect(&rc,MONITOR_DEFAULTTONEAREST);
	mi.cbSize=sizeof(MONITORINFO);
	GetMonitorInfo(hMonitor,&mi);
	if (rc.left>=mi.rcMonitor.right || rc.top>=mi.rcMonitor.bottom
			|| rc.right<=mi.rcMonitor.left || rc.bottom<=mi.rcMonitor.top) {
		int XOffset,YOffset;

		if (rc.left>=mi.rcMonitor.right)
			XOffset=mi.rcMonitor.right-rc.right;
		else if (rc.right<=mi.rcMonitor.left)
			XOffset=mi.rcMonitor.left-rc.left;
		if (rc.top>=mi.rcMonitor.bottom)
			YOffset=mi.rcMonitor.bottom-rc.bottom;
		else if (rc.bottom<=mi.rcMonitor.top)
			YOffset=mi.rcMonitor.top-rc.top;
		OffsetRect(&rc,XOffset,YOffset);
		SetPosition(rc.left,rc.top,rc.right-rc.left,rc.bottom-rc.top);
		return true;
	}
	return false;
}


bool CBasicWindow::CreateBasicWindow(HWND hwndParent,DWORD Style,DWORD ExStyle,
				int ID,LPCTSTR pszClassName,LPCTSTR pszText,HINSTANCE hinst)
{
	if (m_hwnd!=NULL)
		return false;
	m_hwnd=::CreateWindowEx(ExStyle,pszClassName,pszText,Style,
		m_WindowPosition.Left,m_WindowPosition.Top,
		m_WindowPosition.Width,m_WindowPosition.Height,
		hwndParent,(HMENU)ID,hinst,this);
	return m_hwnd!=NULL;
}


void CBasicWindow::OnDestroy()
{
	GetPosition(&m_WindowPosition.Left,&m_WindowPosition.Top,
				&m_WindowPosition.Width,&m_WindowPosition.Height);
	m_hwnd=NULL;
}
